package reader;

import java.io.PrintStream;
import java.util.*;

public class Display {
    
    //**************************************************************************
    
    // Display text corpus:
    public void corpus(PrintStream diskWriter, ArrayList<Sentence> text) {
        for (int i = 0; i < Reader.NSentences; i++) {
            diskWriter.printf(" Sentence %2d\n", i);
            for (int j = 0; j < text.get(i).numberWords; j++) {
                diskWriter.printf(" %5.0f %4.2f %2.0f %3.0f %4.1f %3.0f" + text.get(i).get(j).iv.letters + "\n", text.get(i).get(j).iv.freq, text.get(i).get(j).iv.cloze,
                        text.get(i).get(j).iv.length, text.get(i).get(j).iv.charPos0, text.get(i).get(j).iv.OVP, text.get(i).get(j).iv.charPosN);
            }
        }
    }
    
    //**************************************************************************
     
    // Display models states:
    public void state(PrintStream diskWriter, int sentence, int N, double currentPosition, double processingRate, ArrayList<Process> active) {
        diskWriter.printf(" Sentence: %2d N: %2d Fix: %3.1f Rate: %3.2f -", sentence, N, currentPosition, processingRate);
        for (int i = 0; i < active.size(); i++) if (active.get(i).name.equals("V")) diskWriter.printf("  V %3.0f", active.get(i).duration);
        for (int i = 0; i < active.size(); i++) if (active.get(i).name.equals("L1")) diskWriter.printf(" L1 %3.0f", active.get(i).duration);
        for (int i = 0; i < active.size(); i++) if (active.get(i).name.equals("L2")) diskWriter.printf(" L2 %3.0f", active.get(i).duration);
        for (int i = 0; i < active.size(); i++) if (active.get(i).name.equals("I")) diskWriter.printf("  I %3.0f", active.get(i).duration);
        for (int i = 0; i < active.size(); i++) if (active.get(i).name.equals("A")) diskWriter.printf("  A %3.0f", active.get(i).duration);
        for (int i = 0; i < active.size(); i++) if (active.get(i).name.equals("M1")) diskWriter.printf(" M1 %3.0f (%1d %2.1f)", active.get(i).duration, active.get(i).word, active.get(i).saccadeLength);
        for (int i = 0; i < active.size(); i++) if (active.get(i).name.equals("M2")) diskWriter.printf(" M2 %3.0f (%1d %2.1f)", active.get(i).duration, active.get(i).word, active.get(i).saccadeLength);
        for (int i = 0; i < active.size(); i++) if (active.get(i).name.equals("S")) diskWriter.printf("  S %3.0f (%1d)", active.get(i).duration, active.get(i).word);
        diskWriter.println();
    }
    
    //**************************************************************************
     
    // Display durations of M1 programming components:
    public void M1Programs(PrintStream diskWriter, double priorM1Duration, double priorM1DurationCopy, double currentM1Duration) {
        diskWriter.printf(" Prior M1 duration: %3.0f copy: %3.0f elapsed: %3.0f - Current M1 duration: %3.0f\n", priorM1Duration, priorM1DurationCopy, 
                (priorM1DurationCopy - priorM1Duration), currentM1Duration);
    }
    
    //**************************************************************************
     
    // Display trace of fixation durations & locations:
    public void fixations(PrintStream diskWriter, ArrayList<Sentence> text, int S, ArrayList<Fixation> trace) {
        for (int i = 0; i < trace.size(); i++) {
            diskWriter.printf(" dur: %3.0f pos: %4.1f word: %2d ", trace.get(i).dur, trace.get(i).pos, trace.get(i).word);
            diskWriter.printf(text.get(S).get(trace.get(i).word).iv.letters);
            diskWriter.println();
        }
    }
    
    //**************************************************************************
     
    // Display word-based means:
    public void displayWordBasedMeasures(PrintStream diskWriter, ArrayList<Sentence> text, int S) {
        diskWriter.println();       
        for (int j = 0; j < text.get(S).numberWords; j++) {
            diskWriter.printf(" SFD %3.0f FFD %3.0f GD %3.0f TT %3.0f PrF %3.2f Pr1 %3.2f Pr2 %3.2f PrS %3.2f" + text.get(S).get(j).iv.letters + 
                    "\n", text.get(S).get(j).dv.SFD, text.get(S).get(j).dv.FFD, text.get(S).get(j).dv.GD, text.get(S).get(j).dv.TT, 
                    text.get(S).get(j).dv.PrF, text.get(S).get(j).dv.Pr1, text.get(S).get(j).dv.Pr2, text.get(S).get(j).dv.PrS);
        }
    }
    
    //**************************************************************************
     
    // Display word-based means:
    public void wordBasedMeans(PrintStream diskWriter, ArrayList<Sentence> text) {
        diskWriter.println();
        for (int i = 0; i < Reader.NSentences; i++) {
            for (int j = 0; j < text.get(i).numberWords; j++) {
                diskWriter.printf(" SFD %3.0f FFD %3.0f GD %3.0f TT %3.0f PrF %3.2f Pr1 %3.2f Pr2 %3.2f PrS %3.2f" + text.get(i).get(j).iv.letters + 
                        "\n", text.get(i).get(j).dv.SFD, text.get(i).get(j).dv.FFD, text.get(i).get(j).dv.GD, text.get(i).get(j).dv.TT, 
                        text.get(i).get(j).dv.PrF, text.get(i).get(j).dv.Pr1, text.get(i).get(j).dv.Pr2, text.get(i).get(j).dv.PrS);
            }
        }
    }
    
    //**************************************************************************
     
    // Dipslay word-based means for 5 frequency classes of words:
    public void classMeans(PrintStream diskWriter, ArrayList<Word> classMeans) { 
        diskWriter.printf(" Frequency class word-based means:\n   SFD FFD GD  Pr1  Pr2  PrS\n");
        for (int i = 0; i < 5; i++) diskWriter.printf(" %d %3.0f %3.0f %3.0f %3.2f %3.2f %3.2f\n", i + 1, classMeans.get(i).dv.SFD, 
                classMeans.get(i).dv.FFD, classMeans.get(i).dv.GD, classMeans.get(i).dv.Pr1, classMeans.get(i).dv.Pr2, classMeans.get(i).dv.PrS);
    }
    
    //**************************************************************************
     
    // Display mean fixation landing-site distributions for individual words:
    public void fixDists(PrintStream diskWriter, ArrayList<Sentence> text) {
    
        diskWriter.println();
        for (int S = 0; S < Reader.NSentences; S++) {
            for (int j = 1; j < text.get(S).numberWords - 1; j++) {   
                for (int k = 0; k <= text.get(S).get(j).iv.length; k++) diskWriter.printf(" %3.2f", text.get(S).get(j).dv.FixDist[k]);
                diskWriter.printf(" " + text.get(S).get(j).iv.letters);
                diskWriter.println();
            }
        }
    }
    
    //**************************************************************************
    
    // Display mean refixation-probability distributions
    public void refixProbs(PrintStream diskWriter, ArrayList<Sentence> text) {
        
        diskWriter.println();
        for (int S = 0; S < Reader.NSentences; S++) {
            for (int j = 1; j < text.get(S).numberWords - 1; j++) {
                for (int k = 0; k <= text.get(S).get(j).iv.length; k++) diskWriter.printf(" %3.2f", text.get(S).get(j).dv.RefixProb[k]);
                diskWriter.printf(" " + text.get(S).get(j).iv.letters);
                diskWriter.println();
            }
        }
    }
    
    //**************************************************************************
    
    // Display mean IOVP distributions
    public void IOVPs(PrintStream diskWriter, ArrayList<Sentence> text) {
        
        diskWriter.println();
        for (int S = 0; S < Reader.NSentences; S++) {
            for (int j = 1; j <= text.get(S).numberWords - 1; j++) {
                for (int k = 0; k < text.get(S).get(j).iv.length; k++) diskWriter.printf(" %4.0f", text.get(S).get(j).dv.IOVP[k]);
                diskWriter.printf(" " + text.get(S).get(j).iv.letters);
                diskWriter.println();
            }
        }
    }
    
    //**************************************************************************
     
    // Display mean first-fixation landing-site distributions for words of each length:
    public void meanFixDists(PrintStream diskWriter, ArrayList<Word> fixDists) {
        diskWriter.println();
        diskWriter.printf(" First-fixation landing-site distributions:\n");
        for (int i = 1; i < Reader.maxWordLength - 1; i++) {
            diskWriter.printf(" %2d-letter: ", i);
            for (int j = 0; j <= i; j++) diskWriter.printf(" %4.2f", fixDists.get(i).dv.FixDist[j]);
            diskWriter.println();
        }
        diskWriter.println();
    }
    
    //**************************************************************************
    
    // Display mean refixation landing-site distributions for words of each length:
    public void meanRefixProbs(PrintStream diskWriter, ArrayList<Word> fixDists) {
        diskWriter.println();
        diskWriter.printf(" Refixation probability distributions:\n");
        for (int i = 1; i < Reader.maxWordLength - 1; i++) {
            diskWriter.printf(" %2d-letter: ", i);
            for (int j = 0; j <= i; j++) diskWriter.printf(" %4.2f", fixDists.get(i).dv.RefixProb[j]);
            diskWriter.println();
        }
        diskWriter.println();
    }
    
    //**************************************************************************
    
    public void meanIOVPs(PrintStream diskWriter, ArrayList<Word> IOVPs) {
        diskWriter.println();
        diskWriter.printf(" IOVP distributions:\n");
        for (int i = 1; i < Reader.maxWordLength - 1; i++) {
            diskWriter.printf(" %2d-letter: ", i);
            for (int j = 0; j <= i; j++) diskWriter.printf(" %4.0f", IOVPs.get(i).dv.IOVP[j]);
            diskWriter.println();
        }
        diskWriter.println();
    }
}
