// The class instiates all of the processes embedded within E-Z Reader.

package reader;

import java.io.PrintStream;
import java.util.*;

public class Process {
    
    double duration;
    double durationCopy; // Note: Copy is not decremented.
    String name;
    double saccadeLength;
    int word;
    
    // Initialize classes:
    Display display = new Display();
    Gamma g = new Gamma();
    Random r = new Random();
    Saccade saccade = new Saccade();
                
    //**************************************************************************
    // PRE-ATTENTIVE VISUAL PROCESSING
    //**************************************************************************
    
    void initializeV(int N) {
        
        name = "V";
        word = N;
        
        // Calculate duration:
        duration = Reader.V;
    }
    
    //**************************************************************************
    // FAMILIARITY CHECK
    //**************************************************************************
    
    void initializeL1(ArrayList<Sentence> text, int S, int N, boolean ongoingI) {
        
        name = "L1";
        word = N;

         // Determine if word will be predictable:
        double clozeValue;
        if (ongoingI == true) clozeValue = 1.0;
        else clozeValue = 0;
        
        // Calculate duration:
        double PrGuess = r.nextDouble();
        if (PrGuess < (text.get(S).get(N).iv.cloze)) duration = 0;
        else {
            double mu = Reader.Alpha1 - (Reader.Alpha2 * text.get(S).get(N).iv.lnFreq) - (Reader.Alpha3 * (text.get(S).get(N).iv.cloze * clozeValue));
            duration = g.nextDouble(mu, Reader.SigmaGamma);
        }
    }
    
    //**************************************************************************
    // LEXICAL ACCESS
    //**************************************************************************

    void initializeL2(ArrayList<Sentence> text, int S, int N, boolean ongoingI) {
        
        name = "L2";
        word = N;
        
        // Determine if word will be predictable:
        double clozeValue;
        if (ongoingI == true) clozeValue = 1.0;
        else clozeValue = 0;
        
        // Calculate duration:
        double mu = Reader.Alpha1 - (Reader.Alpha2 * text.get(S).get(N).iv.lnFreq) - (Reader.Alpha3 * (text.get(S).get(N).iv.cloze* clozeValue));
        mu *= Reader.Delta;
        duration = g.nextDouble(mu, Reader.SigmaGamma);
    }
    
    //**************************************************************************
    // POST-LEXICAL INTEGRATION
    //**************************************************************************
    
    void initializeI(int N) {
        
        name = "I";
        word = N;
        
        // Calculate duration:
        duration = g.nextDouble(Reader.I, Reader.SigmaGamma);
    }
    
    //**************************************************************************
    // ATTENTION SHIFT
    //**************************************************************************
    
    void initializeA() {
        
        name = "A";

        // Calculate duration:
        duration = g.nextDouble(Reader.A, Reader.SigmaGamma);
    }
    
    //**************************************************************************
    // LABILE SACCADIC PROGRAMMING
    //**************************************************************************
    
    void initializeM1(ArrayList<Process> active, double currentPos, double targetPos, int N) {
        
        name = "M1";
        word = N;

        // Calculate intended saccade length:
        saccadeLength = targetPos - currentPos;
        
        // Calculate duration:
        duration = g.nextDouble(Reader.M1, Reader.SigmaGamma);
        durationCopy = duration;
        
        // Cancel any pending M1 & adjust new M1 duration accordingly:
        for (int i = 0; i < active.size(); i++) if (active.get(i).name.equals("M1")) {
            
            // *** Display M1 durations:
            //display.M1Programs(active.get(i).duration, active.get(i).durationCopy, duration);
            
            double programmingTimeCompleted = active.get(i).durationCopy - active.get(i).duration;
            double engageTime = duration * Reader.Xi;
            if (programmingTimeCompleted > engageTime) duration -= engageTime;
            else duration -= programmingTimeCompleted;
            
            // *** Display M1 durations:
            //display.M1Programs(active.get(i).duration, active.get(i).durationCopy, duration);
            
            active.remove(i);
        }
    }
    
    //**************************************************************************
    // NON-LABILE SACCADIC PROGRAMMING
    //**************************************************************************
    
    void initializeM2(double intendedSaccade, double launchSiteFixDur, int N) {
        
        name = "M2";
        word = N;
        
        // Calculate saccadic error & actual saccade length:
        double randomError = saccade.randomError(intendedSaccade);
        double systematicError = saccade.systematicError(intendedSaccade, launchSiteFixDur);
        saccadeLength = intendedSaccade + randomError + systematicError;
        
        // Calculate duration:
        duration = g.nextDouble(Reader.M2, Reader.SigmaGamma);
    }
    
    //**************************************************************************
    // SACCADE EXECUTION
    //**************************************************************************
    
    void initializeS(double actualSaccade, int N) {
        
        name = "S";
        word = N;
        
        // Calculate saccade length:
        saccadeLength = actualSaccade;
        
        // Calculate duration:
        duration = Reader.S;
    }
    
    //**************************************************************************
    // ADJUST LEXICAL PROCESSING RATE
    //**************************************************************************
    
    double calcProcessingRate(ArrayList<Sentence> text, int S, int N, double currentPos) {
        
        // Calculate mean absolute deviation between fixation position and letters of attended word:
        double meanAbsDev = 0;
        for (double i = text.get(S).get(N).iv.charPos1; i <= text.get(S).get(N).iv.charPosN; i++) meanAbsDev += Math.abs(i - currentPos);
        meanAbsDev /= text.get(S).get(N).iv.length;
        
        // Return processing rate:
        return Math.pow(Reader.Epsilon, meanAbsDev);
    }
}
