//******************************************************************************
// E-Z Reader, ver. 10.2; (c) 2016 Erik D. Reichle, Ph.D.; E.Reichle@soton.ac.uk
//******************************************************************************

package reader;

import java.io.*;
import java.util.*;

public class Reader {
    
    // I/O file names:
    public static String corpusFileName = "SRC98Corpus.txt";
    public static String outputFileName = "SimulationResults.txt";
    
    // Free (model) parameters:
    public static double A = 25.0; // 25.0
    public static double Alpha1 = 115.0; // 115
    public static double Alpha2 = 2.2; // 2.2
    public static double Alpha3 = 13.0; // 13.0
    public static double Delta = 0.22; // 0.22
    public static double Epsilon = 1.15; // 1.15
    public static double Eta1 = 0.5; // 0.5
    public static double Eta2 = 0.1; // 0.1
    public static double I = 25.0; // 25.0
    public static double Lambda = 0.25; // 0.25
    public static double M1 = 125.0; // 125.0
    public static double M2 = 25.0; // 25.0
    public static double Omega1 = 6.0; // 6.0
    public static double Omega2 = 3.0; // 3.0
    public static double pF = 0.01; // 0.01
    public static double Psi = 7.0; // 7.0
    public static double S = 25.0; // 25.0
    public static double SigmaGamma = 20.0; // 20.0
    public static double V = 50.0; // 50.0
    public static double Xi = 0.5; // 0.5
    
    // Fixed (simulation) parameters:
    public static boolean includeRegressions = true; // include regressions?
    public static int maxWordLength = 16; // = letters + space left of word
    public static int NSentences = 48;
    public static int NSubjects = 1;
    
    // Toggles:
    public static boolean displayClassMeans = false; // frequency class means
    public static boolean displayCorpus = false; // all words & their IVs
    public static boolean displayDists = false; // fixation distributions
    public static boolean displayModelStates = false; // E-Z Reader states
    public static boolean displayTrace = false; // fixation trace
    public static boolean displayWordBasedMeans = false; // mean word DVs
    public static boolean displayWordBasedMeasures = false; // DVs from traces 
    
    
    public static void main(String[] args) throws FileNotFoundException {
        
        // Initialize classes:
        Display display = new Display();
        Stats stats = new Stats();
        
        // Set up output file:
        PrintStream diskWriter = new PrintStream(outputFileName); 
        
        // Set up sentence corpus:
        Scanner diskScanner1 = new Scanner(new File(Reader.corpusFileName)); 
        ArrayList<Sentence> text = new ArrayList<>();
        Corpus corpus = new Corpus();
        corpus.initialize(diskScanner1, text);
        if (displayCorpus == true) display.corpus(diskWriter, text);
        
        int run = 1;
        
        // The following embedded loops are for parameter space grid searches:       
        for (double var1 = 0; var1 < 0.01; var1 += 1.0) {
            for (double var2 = 0; var2 < 0.01; var2 += 1.0) { 
                for (double var3 = 0; var3 < 0.01; var3 += 1.0) { 
                    for (double var4 = 0; var4 < 0.01; var4 += 1.0) { 
                        for (double var5 = 0; var5 < 0.01; var5 += 1.0) {

                            Utility utility = new Utility();
                            
                            //Alpha1 = var1;
                            //Alpha2 = var2;
                            //Alpha3 = var3;
                            //Delta = var4;
                            //Epislon = var5; 

                            // Run model:
                            EZReader ezreader = new EZReader();
                            ezreader.run(diskWriter, text, utility);

                            // Initialize classes for frequency-class means:
                            ArrayList<Word> classMeans = new ArrayList<>();
                            for (int i = 0; i < 5; i++) {
                                Word classMean = new Word();
                                classMeans.add(classMean);
                            }

                            // Calculate & display frequency class means:
                            stats.calcClassMeans(text, classMeans);
                            if (displayClassMeans == true) display.classMeans(diskWriter, classMeans);
                    
                            // Display parameter values & RMSD:
                            double RMSD = stats.calcRMSD(classMeans);
                            //System.out.printf("\n RMSD = %4.3f\n", RMSD);
                            if (displayClassMeans == true) diskWriter.printf("\n RMSD = %4.3f\n", RMSD);
                            //System.out.printf(" %d %3.2f %3.2f %3.2f %3.2f %3.2f %5.4f\n", run, var1, var2, var3, var4, var5, RMSD);
                    
                            if (displayDists == true) {
                        
                                // Initialize classes for mean first-fixation landing-site distributions:
                                ArrayList<Word> fixDists = new ArrayList<>();
                                for (int i = 0; i < maxWordLength; i++) {
                                    Word fixDist = new Word();
                                    fixDists.add(fixDist);
                                }
                    
                                // Calculate & display mean first-fixation landing-site distributions:
                                stats.calcFixDists(text, fixDists);
                                display.meanFixDists(diskWriter, fixDists);
                                fixDists.clear();
                        
                                // Initialize classes for mean refixation-probability distributions:
                                ArrayList<Word> refixProbs = new ArrayList<>();
                                for (int i = 0; i < maxWordLength; i++) {
                                    Word refixProb = new Word();
                                    refixProbs.add(refixProb);
                                }
                    
                                // Calculate & display mean refixation-probability distributions:
                                stats.calcRefixProbs(text, refixProbs);
                                display.meanRefixProbs(diskWriter, refixProbs); 
                                
                                // Initialize classes for mean IOVPs:
                                ArrayList<Word> IOVPs = new ArrayList<>();
                                for (int i = 0; i < maxWordLength; i++) {
                                    Word IOVP = new Word();
                                    IOVPs.add(IOVP);
                                }
                    
                                // Calculate & display mean refixation-probability distributions:
                                stats.calcIOVPs(text, IOVPs);
                                display.meanIOVPs(diskWriter, IOVPs);        
                            
                            }

                            // Re-initialize DVs:
                            corpus.zeroDVs(text);
                    
                            // Increment simulation #:
                            run++; 
                        }                 
                    } 
                }
            }
        }
        
    }
}
